# This file is part of EDIV package
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation
# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Copyright: (C) 2008 Telefonica Investigacion y Desarrollo, S.A.U. 
#                2011 Dpto. Ingenieria de Sistemas Telematicos - UPM
# Authors: Fco. Jose Martin,
#          Miguel Ferrer
#		   David Fernandez
#          Departamento de Ingenieria de Sistemas Telematicos, Universidad Politécnica de Madrid
#

package VNX::ClusterConfig;

use strict;
use warnings;
use Exporter;
use AppConfig;         					# Config files management library
use AppConfig qw(:expand :argcount);    # AppConfig module constants import
use Socket;								# To resolve hostnames to IPs
use VNX::Globals;

our @ISA    = qw(Exporter);
our @EXPORT = qw(
    $cluster_conf_file
	$db
	$vlan
    $cluster
    @cluster_hosts
    read_cluster_config
);

###########################################################
# Global objects

# Cluster configuration file
our $cluster_conf_file;	

# Database configuration
our $db = {
    name => undef,
    type => undef,
    host => undef,
    port => undef,
    user => undef,
    pass => undef,
    conn_info => undef,
};

# Vlan configuration
our $vlan = {
    first => undef,
    last  => undef,
};

# Cluster hosts configuration
our @cluster_hosts;	
our $cluster = {
    hosts         => { my %hosts },
    def_seg_alg   => undef,
    mgmt_net      => undef,
    mgmt_net_mask => undef,
    
};

#
# read_cluster_config
#
# Parse configuration file and load the information on $db, $vlan and $cluster 
# global objects
# 
# Arguments:
# - none
#
# Returns:
# - '' if OK or a string with an error description in case of error 
# 
sub read_cluster_config {

	my $error_management = sub  {
		my $error_msg = shift;
		print "ERROR: when processing $cluster_conf_file file\n       " . $error_msg . "\n";
    };

	my $cluster_config = AppConfig->new(
		{
			CASE  => 0,                     # Case insensitive
			ERROR => $error_management,    # Error control function
			CREATE => 1,    				# Variables that weren't previously defined, are created
			GLOBAL => {
				DEFAULT  => "<unset>",		# Default value for all variables
				ARGCOUNT => ARGCOUNT_ONE,	# All variables contain a single value...
			},
			PEDANTIC => 1
		}
	);
	# Create corresponding cluster host objects
	$cluster_config->define( "cluster_host", { ARGCOUNT => ARGCOUNT_LIST } );	# ...except [CLUSTER] => host
	my $res = $cluster_config->file($cluster_conf_file);		# read the default cluster config file
	unless ($res == 1) {
		return "Error loading $cluster_conf_file cluster configuration file";
	}
	# TODO: check return code 
	
	# Fill database access data  
	#$db->{name} = $cluster_config->get("db_name"); unless ($db->{name}) { return "db_name configuration parameter not found"}
	unless ( $db->{name} = $cluster_config->get("db_name") ) 
		{ return "'name' configuration parameter not found in section [db]"}
	unless ( $db->{type} = $cluster_config->get("db_type") ) 
		{ return "'type' configuration parameter not found in section [db]"}
	unless ( $db->{host} = $cluster_config->get("db_host") ) 
		{ return "'host' configuration parameter not found in section [db]"}
	unless ( $db->{port} = $cluster_config->get("db_port") ) 
		{ return "'port' configuration parameter not found in section [db]"}
	unless ( $db->{user} = $cluster_config->get("db_user") ) 
		{ return "'user' configuration parameter not found in section [db]"}
	unless ( $db->{pass} = $cluster_config->get("db_pass") ) 
		{ return "'pass' configuration parameter not found in section [db]"}
	unless ( $db->{conn_info} = "DBI:$db->{type}:database=$db->{name};$db->{host}:$db->{port}" ) 
	   	{ return "'conn_info' configuration parameter not found in section [db]"}	

	# Fill vlan range data 
	unless ( defined ($vlan->{first} = $cluster_config->get("vlan_first") ) )
		{ return "'first' configuration parameter not found in section [vlan]"}
	unless ( defined ($vlan->{last}  = $cluster_config->get("vlan_last") ) )
		{ return "'last' configuration parameter not found in section [vlan]"}

	my $phy_hosts = ( $cluster_config->get("cluster_host") ); # gets all hosts entries under [cluster]	
	
	#print "Host num: " . scalar(@$phy_hosts) . "\n";
	if ( scalar(@$phy_hosts) == 0 ) { return "No hosts defined in $cluster_conf_file [cluster] section"}
	
	# For each host in the cluster
	foreach my $current_host (@$phy_hosts) {

		# Read current host settings from config file
		# Get hostname
		my $cfg_host_name;
		unless ( $cfg_host_name = $cluster_config->get("$current_host"."_hostname") )
			{ return "'hostname' configuration parameter not found in section [$current_host]";	}
		my $packed_ip = gethostbyname($cfg_host_name);
		unless ( defined($packed_ip) ) {
			return "ERROR: cannot get IP address for host $current_host";
		}
		my $ip = inet_ntoa($packed_ip);
		my $hostname = gethostbyaddr($packed_ip, AF_INET);
		if ($hostname eq '') { $hostname = $ip }
		
		my $mem;
		unless ( $mem = $cluster_config->get("$current_host"."_mem") )
			{ return "'mem' configuration parameter not found in section [$current_host]";	}
		
		my $cpu;
		unless ( $cpu = $cluster_config->get("$current_host"."_cpu") )
			{ return "'cpu' configuration parameter not found in section [$current_host]";	}

		my $max_vhost;
		unless ( defined ($max_vhost = $cluster_config->get("$current_host"."_max_vhost")) )
			{ return "'max_vhost' configuration parameter not found in section [$current_host]";	}
		
		my $ifname;
		unless ( $ifname = $cluster_config->get("$current_host"."_ifname") )
			{ return "'ifname' configuration parameter not found in section [$current_host]";	}
		
		my $cpu_dynamic_command = 'cat /proc/loadavg | awk \'{print $1}\'';
		my $cpu_dynamic = `ssh -2 -o 'StrictHostKeyChecking no' -X root\@$ip $cpu_dynamic_command`;
		chomp $cpu_dynamic;
			
		# Get vnx_dir for each host in the cluster 
		my $vnxDir = `ssh -X -2 -o 'StrictHostKeyChecking no' root\@$ip 'cat /etc/vnx.conf | grep ^vnx_dir'`;
		if ($vnxDir eq '') { 
			$vnxDir = $DEFAULT_VNX_DIR
		} else {
			my @aux = split(/=/, $vnxDir);
			chomp($aux[1]);
			$vnxDir=$aux[1];
		}	
			
		# Create new cluster host object
		my $cluster_host = eval { new_host VNX::ClusterConfig(); } or die ($@);
			
		# Fill cluster host object with parsed data
		$cluster_host->hostName("$hostname");
		$cluster_host->ipAddress("$ip");
		$cluster_host->mem("$mem");
		$cluster_host->cpu("$cpu");
		$cluster_host->maxVhost("$max_vhost");
		$cluster_host->ifName("$ifname");
		$cluster_host->cpuDynamic("$cpu_dynamic");
		$cluster_host->vnxDir("$vnxDir");
		
		# Store the host object inside cluster arrays
		$cluster->{hosts}{$current_host} = $cluster_host;
		push(@cluster_hosts, $current_host);

	}

	$cluster->{def_seg_alg} = $cluster_config->get("cluster_default_segmentation");
	$cluster->{mgmt_network} = $cluster_config->get("cluster_mgmt_network");
	$cluster->{mgmt_network_mask} = $cluster_config->get("cluster_mgmt_network_mask");

	return;	
}

#
# Constructor for host information records
#
sub new_host {
    my ($class) = @_;
    
    my $self = {
        _hostname => undef,		# Name of host
        _ipaddress => undef,	# IP Address
        _mem => undef, 			# RAM MegaBytes
        _cpu => undef,			# Percentage of CPU speed
        _maxvhost => undef,		# Maximum virtualized host (0 = unlimited)
        _ifname => undef,		# Network interface of the physical host
        _cpudynamic => undef,	# CPU load in present time
        _vnxdir => undef     	# VNX directory  
    };
    bless $self, $class;
    return $self;
}
	
#
# Accessor methods for the host record fields
#
sub cpuDynamic {
    my ( $self, $cpudynamic ) = @_;
    $self->{_cpudynamic} = $cpudynamic if defined($cpudynamic);
    return $self->{_cpudynamic};
}

sub hostName {
    my ( $self, $hostName ) = @_;
    $self->{_hostname} = $hostName if defined($hostName);
    return $self->{_hostname};
}

sub ipAddress {
    my ( $self, $ipAddress ) = @_;
    $self->{_ipaddress} = $ipAddress if defined($ipAddress);
    return $self->{_ipaddress};
}        

sub mem {
    my ( $self, $Mem ) = @_;
    $self->{_mem} = $Mem if defined($Mem);
    return $self->{_mem};
}

sub cpu {
    my ( $self, $CPU ) = @_;
    $self->{_cpu} = $CPU if defined($CPU);
    return $self->{_cpu};
}

sub maxVhost {
    my ( $self, $maxVhost ) = @_;
    $self->{_maxvhost} = $maxVhost if defined($maxVhost);
    return $self->{_maxvhost};
}

sub ifName {
    my ( $self, $if ) = @_;
    $self->{_ifname} = $if if defined($if);
    return $self->{_ifname};
}

sub vnxDir {
    my ( $self, $vnxdir ) = @_;
    $self->{_vnxdir} = $vnxdir if defined($vnxdir);
    return $self->{_vnxdir};
}

1;
###########################################################
